@@ -153,40 +153,33 @@ Servers does not need to activate the effect flag recording. Effect flags that
 you create will not cause interference with other clients or servers without
 the effect flag recording.
 
-Templates
-=========
+Template keywords
+=================
 
-Evolve ship several templates that you can use to have a better visibility
-about your obs history:
+Evolve provides one template keyword that helps explore obsolescence history:
 
-  - precursors, for each obsolete changeset show the closest visible
-    precursors.
-  - successors, for each obsolete changeset show the closest visible
-    successors. It is useful when your working directory is obsolete to see
-    what are its successors. This information can also be retrieved with the
-    obslog command and the --all option.
-  - obsfate, for each obsolete changeset display a line summarizing what
-    changed between the changeset and its successors. Depending on the
-    verbosity level (-q and -v) it display the changeset successors, the users
-    that created the obsmarkers and the date range of these changes.
+  - obsorigin, for each changeset display a line summarizing what changed
+    between the changeset and its predecessors. Depending on the verbosity
+    level (-q and -v) it displays the users that created the obsmarkers and the
+    date range of these operations.
 
-    The template itself is not complex, the data are basically a list of
-    successortset. Each successorset is a dict with these fields:
+Evolve used to provide these template keywords, which since have been included
+in core Mercurial (see :hg:`help templates -v`):
 
-      - "verb", how did the revision changed, pruned or rewritten for the moment
-      - "users" a sorted list of users that have create obs marker between current
-        changeset and one of its successor
-      - "min_date" the tiniest date of the first obs marker between current
-        changeset and one of its successor
-      - "max_date" the biggest date between current changeset and one of its
-        successor
-      - "successors" a sorted list of locally know successors node ids
-      - "markers" the raw list of changesets.
+  - obsolete
+  - obsfate (including obsfatedata, see also obsfate* template functions)
+
+For compatibility, this extension also provides the following aliases to
+template keywords from core Mercurial:
+
+  - precursors (deprecated, use predecessors instead)
+  - successors (deprecated, use successorssets instead)
+  - troubles (deprecated, use instabilities instead)
 """
 
 evolutionhelptext = b"""
 Obsolescence markers make it possible to mark changesets that have been
-deleted or superset in a new version of the changeset.
+deleted or superseded in a new version of the changeset.
 
 Unlike the previous way of handling such changes, by stripping the old
 changesets from the repository, obsolescence markers can be propagated
@@ -374,7 +367,7 @@ def _configureoptions(ui, repo):
     if obsolete.isenabled(repo, b'exchange'):
         # if no config explicitly set, disable bundle1
         if not isinstance(repo.ui.config(b'server', b'bundle1'), bytes):
-            repo.ui.setconfig(b'server', b'bundle1', False)
+            repo.ui.setconfig(b'server', b'bundle1', False, b'evolve')
 
     class trdescrepo(repo.__class__):
 
