@@ -169,11 +169,11 @@ def amendpatch(ui, repo, *pats, **opts):
         metadata = {}
         if opts.get('note'):
             metadata[b'note'] = opts['note']
-        replacements = {old.node(): [newnode]}
-        scmutil.cleanupnodes(repo, replacements, operation=b'amend',
-                             metadata=metadata)
+        replacements = {(old.node(),): [newnode]}
+        compat.cleanupnodes(repo, replacements, operation=b'amend',
+                            metadata=metadata)
         phases.retractboundary(repo, tr, old.phase(), [newnode])
-        hg.updaterepo(repo, newnode, True)
+        compat.clean_update(repo[newnode])
 
 def _editandapply(ui, repo, pats, old, p1, fp, diffopts):
     newnode = None
@@ -341,100 +341,49 @@ def _commitfiltered(repo, ctx, match, target=None, message=None, user=None,
     newid = repo.commitctx(new)
     return newid
 
-def _uncommitdirstate(repo, oldctx, match, interactive):
-    """Fix the dirstate after switching the working directory from
-    oldctx to a copy of oldctx not containing changed files matched by
-    match.
-    """
-    ctx = repo[b'.']
-    ds = repo.dirstate
-    copies = dict(ds.copies())
-    if interactive:
-        # In interactive cases, we will find the status between oldctx and ctx
-        # and considering only the files which are changed between oldctx and
-        # ctx, and the status of what changed between oldctx and ctx will help
-        # us in defining the exact behavior
-        st = repo.status(oldctx, ctx, match=match)
-        for f in st.modified:
-            # These are files which are modified between oldctx and ctx which
-            # contains two cases: 1) Were modified in oldctx and some
-            # modifications are uncommitted
-            # 2) Were added in oldctx but some part is uncommitted (this cannot
-            # contain the case when added files are uncommitted completely as
-            # that will result in status as removed not modified.)
-            # Also any modifications to a removed file will result the status as
-            # added, so we have only two cases. So in either of the cases, the
-            # resulting status can be modified or clean.
-            if ds[f] == b'r':
-                # But the file is removed in the working directory, leaving that
-                # as removed
-                continue
-            ds.normallookup(f)
+# TODO: call core's version once we've dropped support for hg <= 4.9
+def movedirstate(repo, newctx, match=None):
+    """Move the dirstate to newctx and adjust it as necessary.
 
-        for f in st.added:
-            # These are the files which are added between oldctx and ctx(new
-            # one), which means the files which were removed in oldctx
-            # but uncommitted completely while making the ctx
-            # This file should be marked as removed if the working directory
-            # does not adds it back. If it's adds it back, we do a normallookup.
-            # The file can't be removed in working directory, because it was
-            # removed in oldctx
-            if ds[f] == b'a':
-                ds.normallookup(f)
-                continue
+    A matcher can be provided as an optimization. It is probably a bug to pass
+    a matcher that doesn't match all the differences between the parent of the
+    working copy and newctx.
+    """
+    oldctx = repo[b'.']
+    ds = repo.dirstate
+    dscopies = dict(ds.copies())
+    ds.setparents(newctx.node(), node.nullid)
+    s = newctx.status(oldctx, match=match)
+    for f in s.modified:
+        if ds[f] == b'r':
+            # modified + removed -> removed
+            continue
+        ds.normallookup(f)
+
+    for f in s.added:
+        if ds[f] == b'r':
+            # added + removed -> unknown
+            ds.drop(f)
+        elif ds[f] != b'a':
+            ds.add(f)
+
+    for f in s.removed:
+        if ds[f] == b'a':
+            # removed + added -> normal
+            ds.normallookup(f)
+        elif ds[f] != b'r':
             ds.remove(f)
 
-        for f in st.removed:
-            # These are files which are removed between oldctx and ctx, which
-            # means the files which were added in oldctx and were completely
-            # uncommitted in ctx. If a added file is partially uncommitted, that
-            # would have resulted in modified status, not removed.
-            # So a file added in a commit, and uncommitting that addition must
-            # result in file being stated as unknown.
-            if ds[f] == b'r':
-                # The working directory say it's removed, so lets make the file
-                # unknown
-                ds.drop(f)
-                continue
-            ds.add(f)
-    else:
-        st = repo.status(oldctx.p1(), oldctx, match=match)
-        for f in st.modified:
-            if ds[f] == b'r':
-                # modified + removed -> removed
-                continue
-            ds.normallookup(f)
-
-        for f in st.added:
-            if ds[f] == b'r':
-                # added + removed -> unknown
-                ds.drop(f)
-            elif ds[f] != b'a':
-                ds.add(f)
-
-        for f in st.removed:
-            if ds[f] == b'a':
-                # removed + added -> normal
-                ds.normallookup(f)
-            elif ds[f] != b'r':
-                ds.remove(f)
-
     # Merge old parent and old working dir copies
-    oldcopies = {}
-    if interactive:
-        # Interactive had different meaning of the variables so restoring the
-        # original meaning to use them
-        st = repo.status(oldctx.p1(), oldctx, match=match)
-    for f in (st.modified + st.added):
-        src = oldctx[f].renamed()
-        if src:
-            oldcopies[f] = src[0]
-    oldcopies.update(copies)
-    copies = dict((dst, oldcopies.get(src, src))
-                  for dst, src in oldcopies.items())
+    oldcopies = copies.pathcopies(newctx, oldctx, match)
+    oldcopies.update(dscopies)
+    newcopies = {
+        dst: oldcopies.get(src, src)
+        for dst, src in oldcopies.items()
+    }
     # Adjust the dirstate copies
-    for dst, src in copies.items():
-        if (src not in ctx or dst in ctx or ds[dst] != b'a'):
+    for dst, src in newcopies.items():
+        if src not in newctx or dst in newctx or ds[dst] != b'a':
             src = None
         ds.copy(src, dst)
 
@@ -559,16 +508,15 @@ def uncommit(ui, repo, *pats, **opts):
         if opts.get('note'):
             metadata[b'note'] = opts['note']
 
-        replacements = {old.node(): [newid]}
-        scmutil.cleanupnodes(repo, replacements, operation=b"uncommit",
-                             metadata=metadata)
+        replacements = {(old.node(),): [newid]}
+        compat.cleanupnodes(repo, replacements, operation=b"uncommit",
+                            metadata=metadata)
         phases.retractboundary(repo, tr, oldphase, [newid])
         if opts.get('revert'):
-            hg.updaterepo(repo, newid, True)
+            compat.clean_update(repo[newid])
         else:
             with repo.dirstate.parentchange(), compat.parentchange(repo):
-                repo.dirstate.setparents(newid, node.nullid)
-                _uncommitdirstate(repo, old, match, interactive)
+                movedirstate(repo, repo[newid], match)
         if not repo[newid].files():
             ui.warn(_(b"new changeset is empty\n"))
             ui.status(_(b"(use 'hg prune .' to remove it)\n"))
@@ -797,17 +745,9 @@ def fold(ui, repo, *revs, **opts):
                                                          p2.node()],
                                                         commitopts=commitopts)
             phases.retractboundary(repo, tr, targetphase, [newid])
-            # Use this condition as a proxy since the commit we care about
-            # (b99903534e06) didn't change any signatures.
-            if util.safehasattr(scmutil, 'nullrev'):
-                replacements = {tuple(ctx.node() for ctx in allctx): [newid]}
-                scmutil.cleanupnodes(repo, replacements, operation=b"fold",
-                                     metadata=metadata)
-            else:
-                # hg <= 4.7 (b99903534e06)
-                replacements = {ctx.node(): [newid] for ctx in allctx}
-                scmutil.cleanupnodes(repo, replacements, operation=b"fold",
-                                     metadata=metadata)
+            replacements = {tuple(ctx.node() for ctx in allctx): [newid]}
+            compat.cleanupnodes(repo, replacements, operation=b"fold",
+                                metadata=metadata)
             tr.close()
         finally:
             tr.release()
@@ -935,14 +875,16 @@ def metaedit(ui, repo, *revs, **opts):
 
                 phases.retractboundary(repo, tr, targetphase, [newid])
                 # Use this condition as a proxy since the commit we care about
-                # (6335c0de80fa) didn't change any signatures.
-                if not util.safehasattr(context, 'bin'):
-                    obsolete.createmarkers(repo, [(tuple(allctx), (repo[newid],))],
-                                           metadata=metadata, operation=b"metaedit")
+                # (b99903534e06) didn't change any signatures.
+                if util.safehasattr(scmutil, 'nullrev'):
+                    mapping = {tuple(ctx.node() for ctx in allctx): (newid,)}
+                    scmutil.cleanupnodes(repo, mapping, operation=b"metaedit",
+                                         metadata=metadata)
                 else:
-                    # hg <= 4.7 (6335c0de80fa)
-                    obsolete.createmarkers(repo, [(ctx, (repo[newid],)) for ctx in allctx],
-                                           metadata=metadata, operation=b"metaedit")
+                    # hg <= 4.7 (b99903534e06)
+                    mapping = {ctx.node(): (newid,) for ctx in allctx}
+                    scmutil.cleanupnodes(repo, mapping, operation=b"metaedit",
+                                         metadata=metadata)
             else:
                 ui.status(_(b"nothing changed\n"))
             tr.close()
@@ -1081,9 +1023,9 @@ def cmdprune(ui, repo, *revs, **opts):
             msg = b"please add --fold if you want to do a fold"
             raise error.Abort(msg)
         elif biject:
-            replacements = {p.node(): [s.node()] for p, s in zip(precs, sucs)}
+            replacements = {(p.node(),): [s.node()] for p, s in zip(precs, sucs)}
         else:
-            replacements = {p.node(): [s.node() for s in sucs] for p in precs}
+            replacements = {(p.node(),): [s.node() for s in sucs] for p in precs}
 
         wdp = repo[b'.']
 
@@ -1163,16 +1105,13 @@ def cmdprune(ui, repo, *revs, **opts):
             # but then revset took a lazy arrow in the knee and became much
             # slower. The new forms makes as much sense and a much faster.
             for dest in ctx.ancestors():
-                if not dest.obsolete() and dest.node() not in replacements:
+                if not dest.obsolete() and (dest.node(),) not in replacements:
                     moves[ctx.node()] = dest.node()
                     break
         if len(sucs) == 1 and len(precs) > 1 and fold:
-            # hg <= 4.7 (b99903534e06)
-            # Using a proxy condition to let people wrap cleanupnodes()
-            if util.safehasattr(scmutil, 'nullrev'):
-                replacements = {tuple(p.node() for p in precs): [s.node() for s in sucs]}
-        scmutil.cleanupnodes(repo, replacements, operation=b"prune", moves=moves,
-                             metadata=metadata)
+            replacements = {tuple(p.node() for p in precs): [s.node() for s in sucs]}
+        compat.cleanupnodes(repo, replacements, operation=b"prune", moves=moves,
+                            metadata=metadata)
 
         # informs that changeset have been pruned
         ui.status(_(b'%i changesets pruned\n') % len(precs))
@@ -1307,7 +1246,13 @@ def cmdsplit(ui, repo, *pats, **opts):
                         # diff action or by showing the remaining and
                         # prompting for confirmation
                         ui.status(_(b'discarding remaining changes\n'))
-                        target = newcommits[0]
+                        target = newcommits[-1]
+                        args = []
+                        kwargs = {}
+                        code = cmdutil.revert.__code__
+                        # hg <= 5.5 (8c466bcb0879)
+                        if r'parents' in code.co_varnames[:code.co_argcount]:
+                            args.append((target, node.nullid))
                         if pats:
                             status = repo.status(match=matcher)
                             dirty = set()
@@ -1315,12 +1260,10 @@ def cmdsplit(ui, repo, *pats, **opts):
                             dirty.update(status.added)
                             dirty.update(status.removed)
                             dirty.update(status.deleted)
-                            dirty = sorted(dirty)
-                            cmdutil.revert(ui, repo, repo[target],
-                                           (target, node.nullid), *dirty)
+                            args += sorted(dirty)
                         else:
-                            cmdutil.revert(ui, repo, repo[target],
-                                           (target, node.nullid), all=True)
+                            kwargs[r'all'] = True
+                        cmdutil.revert(ui, repo, repo[target], *args, **kwargs)
                     elif nextaction == b'?':
                         nextaction = None
                         ui.write(_(b"y - yes, continue selection\n"))
@@ -1522,11 +1465,11 @@ def cmdpick(ui, repo, *revs, **opts):
         if pickstate:
             pickstate.delete()
         if newnode is None:
-            replacements = {origctx.node(): []}
+            replacements = {(origctx.node(),): []}
         else:
             newctx = repo[newnode]
-            replacements = {origctx.node(): [newctx.node()]}
-        scmutil.cleanupnodes(repo, replacements, operation=b"pick")
+            replacements = {(origctx.node(),): [newctx.node()]}
+        compat.cleanupnodes(repo, replacements, operation=b"pick")
 
         if newnode is None:
             ui.warn(_(b"note: picking %d:%s created no changes to commit\n") %
@@ -1543,7 +1486,7 @@ def abortpick(ui, repo, pickstate, abortcmd=False):
     pctxnode = pickstate[b'oldpctx']
     ui.status(_(b"aborting pick, updating to %s\n") %
               node.hex(pctxnode)[:12])
-    hg.updaterepo(repo, pctxnode, True)
+    compat.clean_update(repo[pctxnode])
     pickstate.delete()
     return 0
 
