@@ -123,6 +123,36 @@ the default in future::
 
     [experimental]
     topic.server-gate-topic-changesets = yes
+
+Explicitly merging in the target branch
+=======================================
+
+By default, Mercurial will not let your merge a topic into its target branch if
+that topic is already based on the head of that branch. In other word,
+Mercurial will not let your create a merge that will eventually have two
+parents in the same branches, one parent being the ancestors of the other
+parent. This behavior can be lifted using the following config::
+
+    [experimental]
+    topic.linear-merge = allow-from-bare-branch
+
+When this option is set to `allow-from-bare-branch`, it is possible to merge a
+topic branch from a bare branch (commit an active topic (eg: public one))
+regardless of the topology. The result would typically looks like that::
+
+   @    summary: resulting merge commit
+   |\\   branch:  my-branch
+   | |
+   | o  summary: some more change in a topic, the merge "target"
+   | |  branch:  my-branch
+   | |  topic:   my-topic
+   | |
+   | o  summary: some change in a topic
+   |/   branch:  my-branch
+   |    topic:   my-topic
+   |
+   o    summary: previous head of the branch, the merge "source"
+   |    branch:  my-branch
 """
 
 from __future__ import absolute_import
@@ -202,9 +232,9 @@ colortable = {b'topic.active': b'green',
               b'topic.active': b'green',
               }
 
-__version__ = b'0.19.1'
+__version__ = b'0.20.0'
 
-testedwith = b'4.6.2 4.7 4.8 4.9 5.0 5.1 5.2 5.3 5.4 5.5'
+testedwith = b'4.6.2 4.7 4.8 4.9 5.0 5.1 5.2 5.3 5.4 5.5 5.6'
 minimumhgversion = b'4.6'
 buglink = b'https://bz.mercurial-scm.org/'
 
@@ -239,6 +269,9 @@ if util.safehasattr(registrar, 'configitem'):
     configitem(b'experimental', b'topic.server-gate-topic-changesets',
                default=False,
     )
+    configitem(b'experimental', b'topic.linear-merge',
+               default="reject",
+    )
 
     def extsetup(ui):
         # register config that strictly belong to other code (thg, core, etc)
@@ -353,7 +386,11 @@ def uisetup(ui):
                      _(b"topic to push"), _(b'TOPIC')))
 
     extensions.wrapfunction(cmdutil, 'buildcommittext', committextwrap)
-    extensions.wrapfunction(merge, 'update', mergeupdatewrap)
+    if util.safehasattr(merge, '_update'):
+        extensions.wrapfunction(merge, '_update', mergeupdatewrap)
+    else:
+        # hg <= 5.5 (2c86b9587740)
+        extensions.wrapfunction(merge, 'update', mergeupdatewrap)
     # We need to check whether t0 or b0 or s0 is passed to override the default update
     # behaviour of changing topic and I can't find a better way
     # to do that as scmutil.revsingle returns the rev number and hence we can't
@@ -1284,7 +1321,37 @@ def mergeupdatewrap(orig, repo, node, branchmerge, force, *args, **kwargs):
     isrebase = False
     ist0 = False
     try:
-        ret = orig(repo, node, branchmerge, force, *args, **kwargs)
+        mergemode = repo.ui.config(b'experimental', b'topic.linear-merge')
+
+        cleanup = lambda: None
+        oldrepo = repo
+        if mergemode == b'allow-from-bare-branch' and not repo[None].topic():
+            unfi = repo.unfiltered()
+            oldrepo = repo
+            old = unfi.__class__
+
+            class overridebranch(old):
+                def __getitem__(self, rev):
+                    ret = super(overridebranch, self).__getitem__(rev)
+                    if rev == node:
+                        b = ret.branch()
+                        t = ret.topic()
+                        if t:
+                            ret.branch = lambda: b'%s//%s' % (b, t)
+                    return ret
+            unfi.__class__ = overridebranch
+            if repo.filtername is not None:
+                repo = unfi.filtered(repo.filtername)
+
+            def cleanup():
+                unfi.__class__ = old
+
+        try:
+            ret = orig(repo, node, branchmerge, force, *args, **kwargs)
+        finally:
+            cleanup()
+            repo = oldrepo
+
         if not hastopicext(repo):
             return ret
         # The mergeupdatewrap function makes the destination's topic as the
